@@ -9,7 +9,7 @@ use Mojo::UserAgent;
 
 use vars qw($DEBUG $CACHE);
 
-our $VERSION = '1.01';
+our $VERSION = '1.02';
 
 our %searches = (
 	cities                              => 'cities?',
@@ -29,6 +29,7 @@ our %searches = (
 	search                              => 'search?',
 	wikipedia_bounding_box              => 'wikipediaBoundingBox?',
 	wikipedia_search                    => 'wikipediaSearch?',
+	get                                 =>  'get?',
 	);
 
 #	r	= required
@@ -57,6 +58,18 @@ our %valid_parameters = (
 		isNameRequired    => 'o',
 		tag    => 'o',
 		username => 'r',
+		name_startsWith => 'o',
+		countryBias => 'o',
+		cities => 'om',
+		operator => 'o',
+		searchlang => 'o',
+		charset => 'o',
+		fuzzy => 'o',
+		north => 'o',
+		west => 'o',
+		east => 'o',
+		south => 'o',
+		orderby => 'o',
 		},
 	postalcode_search => {
 		postalcode    => 'rc',
@@ -168,7 +181,13 @@ our %valid_parameters = (
 		minMagnutide    => 'o',
 		maxRows         => 'o',
 		username        => 'r',
-		}
+		},
+    get => {
+        geonameId => 'r',
+        lang      => 'o',
+        style     => 'o',
+        username  => 'r',
+        },
 	);
 
 sub new {
@@ -209,10 +228,10 @@ sub url {
 	$self->{url};
 	}
 
-sub _build_request {
+sub _build_request_url {
 	my( $self, $request, @args ) = @_;
 	my $hash = { @args, username => $self->username };
-	my $request_string = $self->url . '/' . $searches{$request};
+	my $request_url = $self->url . '/' . $searches{$request};
 
 	# check to see that mandatory arguments are present
 	my $conditional_mandatory_flag = 0;
@@ -241,30 +260,33 @@ sub _build_request {
 		carp("Invalid number of mandatory arguments (there can be only one)");
 		}
 
-	foreach my $key (keys(%$hash)) {
+	foreach my $key (sort keys(%$hash)) {
 		carp("Invalid argument $key") if(!defined($valid_parameters{$request}->{$key}));
 		my @vals = ref($hash->{$key}) ? @{$hash->{$key}} : $hash->{$key};
 		no warnings 'uninitialized';
-		$request_string .= join("", map { "$key=$_&" } @vals );
+		$request_url .= join("", map { "$key=$_&" } sort @vals );
 		}
 
-	chop($request_string); # loose the trailing &
-	return $request_string;
+	chop($request_url); # loose the trailing &
+	return $request_url;
 	}
 
 sub _parse_xml_result {
 	require XML::Simple;
-	my( $self, $geonamesresponse ) = @_;
+	my( $self, $geonamesresponse, $single_result ) = @_;
 	my @result;
 	my $xmlsimple = XML::Simple->new;
 	my $xml = $xmlsimple->XMLin( $geonamesresponse, KeyAttr => [], ForceArray => 1 );
 
+	if ($xml->{'status'}) {
+		carp "GeoNames error: " . $xml->{'status'}->[0]->{message};
+		return [];
+		}
+
+	$xml = { geoname => [ $xml ], totalResultsCount => '1' } if $single_result;
+
 	my $i = 0;
 	foreach my $element (keys %{$xml}) {
-		if ($element eq 'status') {
-			carp "GeoNames error: " . $xml->{$element}->[0]->{message};
-			return [];
-			}
 		next if (ref($xml->{$element}) ne "ARRAY");
 		foreach my $list (@{$xml->{$element}}) {
 			next if (ref($list) ne "HASH");
@@ -329,15 +351,15 @@ sub _request {
 sub _do_search {
 	my( $self, $searchtype, @args ) = @_;
 
-	my $request = $self->_build_request( $searchtype, @args );
-	my $response = $self->_request( $request );
+	my $request_url = $self->_build_request_url( $searchtype, @args );
+	my $response = $self->_request( $request_url );
 
 	# check mime-type to determine which parse method to use.
 	# we accept text/xml, text/plain (how do see if it is JSON or not?)
-	my $mime_type = $response->headers->header( 'Content-type' );
+	my $mime_type = $response->headers->content_type || '';
 
 	if($mime_type =~ m(\Atext/xml;) ) {
-		return $self->_parse_xml_result( $response->body );
+		return $self->_parse_xml_result( $response->body, $searchtype eq 'get' );
 		}
 	if($mime_type =~ m(\Aapplication/json;) ) {
 		# a JSON object always start with a left-brace {
@@ -351,9 +373,9 @@ sub _do_search {
 			}
 		}
 
-	carp "Invalid mime type [$mime_type]";
+	carp "Invalid mime type [$mime_type]. Maybe you aren't connected.";
 
-	return;
+	return [];
 	}
 
 sub geocode {
@@ -380,6 +402,8 @@ sub DESTROY { 1 }
 
 __END__
 
+=encoding utf8
+
 =head1 NAME
 
 Geo::GeoNames - Perform geographical queries using GeoNames Web Services
@@ -814,7 +838,8 @@ at https://github.com/briandfoy/geo-geonames
 
 Per Henrik Johansen, C<< <per.henrik.johansen@gmail.com> >>.
 
-Currently maintained by brian d foy, C<< <brian.d.foy@gmail.com> >>.
+Currently maintained by brian d foy, C<< <brian.d.foy@gmail.com> >> 
+and Nicolas Mendoza, C<< <mendoza@pvv.ntnu.no> >>
 
 =head1 COPYRIGHT AND LICENSE
 
